/* *****************************************************************************
 *
 * Filename: CalendarPanel.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  01 December 2002, 14:43
 *
 * Purpose:  Base class for all CalendarPanels, also a blank calendar template.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker;

import javax.swing.*;
import java.awt.*;
import java.util.ResourceBundle;


public class CalendarPanel
    extends javax.swing.JPanel
    implements IDateable, IScaleable
{
    // Static
    private static final ResourceBundle resources = ResourceBundle.getBundle( "com/tsac/calmaker/CalendarMaker" );
    
    // Properties
    protected int _width;
    protected int _height;

	
    public CalendarPanel()
    {
        setLayout( null );
        setActualSize( 545, 742 );
        setDoubleBuffered( true );
        setFocusable( true );
        
        getAccessibleContext().setAccessibleName( resources.getString( "CalendarPanel_Name" ) );
        getAccessibleContext().setAccessibleDescription( resources.getString( "CalendarPanel_Desc" ) );
    }
	
	
    public int getNumPagesWidth()
    {
        return 1;
    }
    

    // Set the actual size of the bean
    public void setActualSize( Dimension sz )
    {
        this.setActualSize( sz.width, sz.height );
    }

    
    // Set the actual size of the bean
    public void setActualSize( int w, int h )
    {
        _width = w;
        _height = h;
        
        this.setSize( w, h );
        this.doLayout();
    }
    
    
    // Override default to give non-resizable bean
    public Dimension getPreferredSize()
    {
        return new Dimension( _width, _height );
    }
    
    
    // Change date of all nested components
    public void setDate( int year, int month )
    {
        for( int n = 0; n < getComponentCount(); n++ )
        {
            Component c = getComponent( n );

            if( c instanceof IDateable )
                ((IDateable) c).setDate( year, month );
        }
    }
    
    
    /**
     * Overrides default paint to use our scaled paint and the current scale factor.
     */
    public void paint( Graphics g )
    {
        // Fill in background as component is larger than drawing area
        g.setColor( Color.lightGray );
        g.fillRect( 0, 0, this.getWidth(), this.getHeight() );
        
        // Now call the scaled paint rountine
        paint( g, MainFrame.scale );
    }
    
    
    /**
     * Scaled paint routine.  Draws each IScalable component at the position set
     * by its bounds, but scales the co-ordinates and dimensions by the given amount.
     */
    public void paint( Graphics g, double scale )
    {
        g.setColor( Color.white );
        g.fillRect( 0, 0, (int) (scale * getWidth()), (int) (scale * getHeight()) );
        
        for( int n = 0; n < getComponentCount(); n++ )
        {
            Component c = getComponent( n );

            if( c instanceof IScaleable )
            {
                Rectangle bounds = c.getBounds();
                Graphics cg = g.create( (int) (scale * bounds.x), (int) (scale * bounds.y), (int) (scale * bounds.width), (int) (scale * bounds.height) );
                ((IScaleable) c).paint( cg, scale );
                cg.dispose();
            }
        }        
    }
}