/* *****************************************************************************
 *
 * Filename: NullLayoutCustomizer.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  23 December 2002, 11:12
 *
 * Purpose:  Customizer for any container with a null layout.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker;

import java.awt.Component;
import java.awt.Container;
import java.awt.Rectangle;
import java.awt.event.ComponentEvent;
import java.awt.event.FocusEvent;
import java.util.ResourceBundle;
import javax.swing.JComponent;

import com.tsac.calmaker.beans.Utilities;
import com.tsac.calmaker.beans.DateablePanel;

/**
 *
 * @author  Robert
 */
public class NullLayoutCustomizer
    extends javax.swing.JPanel
    implements java.beans.Customizer, java.awt.event.ComponentListener,
               java.awt.event.FocusListener
{
    // Statics
    private static final ResourceBundle calendarMaker = ResourceBundle.getBundle("com/tsac/calmaker/CalendarMaker");
    
    // State
    private Component _object;
    
    /** Creates new form NullLayoutCustomizer */
    public NullLayoutCustomizer()
    {
        initComponents();
        
        xTxt.addFocusListener( this );
        yTxt.addFocusListener( this );
        
        // Accessibility stuff
        constrainBtn.getAccessibleContext().setAccessibleName( calendarMaker.getString( "Constain_Name" ) );
        
        Utilities.setAccessibleLabelNames( this );
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents()//GEN-BEGIN:initComponents
    {
        java.awt.GridBagConstraints gridBagConstraints;

        boundsPnl = new javax.swing.JPanel();
        xLbl = new javax.swing.JLabel();
        xTxt = new javax.swing.JTextField();
        xUnitsLbl = new javax.swing.JLabel();
        yLbl = new javax.swing.JLabel();
        yTxt = new javax.swing.JTextField();
        yUnitsLbl = new javax.swing.JLabel();
        widthLbl = new javax.swing.JLabel();
        widthTxt = new javax.swing.JTextField();
        widthUnitsLbl = new javax.swing.JLabel();
        heightLbl = new javax.swing.JLabel();
        heightTxt = new javax.swing.JTextField();
        heightUnitsLbl = new javax.swing.JLabel();
        constrainBtn = new javax.swing.JToggleButton();

        setLayout(new java.awt.GridBagLayout());

        boundsPnl.setLayout(new java.awt.GridBagLayout());

        boundsPnl.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), calendarMaker.getString("Bounds")));
        xLbl.setText(calendarMaker.getString("X_"));
        xLbl.setDisplayedMnemonic(calendarMaker.getString( "X_Mnemonic" ).charAt(0));
        xLbl.setLabelFor(xTxt);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 2, 2);
        boundsPnl.add(xLbl, gridBagConstraints);

        xTxt.setColumns(6);
        xTxt.setToolTipText(calendarMaker.getString("X_Tooltip"));
        xTxt.getAccessibleContext().setAccessibleDescription( calendarMaker.getString("X_Tooltip") );
        xTxt.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                xTxtActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 2, 2, 2);
        boundsPnl.add(xTxt, gridBagConstraints);

        xUnitsLbl.setText(calendarMaker.getString("pixels"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(4, 2, 2, 4);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        boundsPnl.add(xUnitsLbl, gridBagConstraints);

        yLbl.setText(calendarMaker.getString("Y_"));
        yLbl.setDisplayedMnemonic(calendarMaker.getString( "Y_Mnemonic" ).charAt(0));
        yLbl.setLabelFor(yTxt);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        boundsPnl.add(yLbl, gridBagConstraints);

        yTxt.setColumns(6);
        yTxt.setToolTipText(calendarMaker.getString("Y_Tooltip"));
        yTxt.getAccessibleContext().setAccessibleDescription( calendarMaker.getString("Y_Tooltip") );
        yTxt.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                yTxtActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        boundsPnl.add(yTxt, gridBagConstraints);

        yUnitsLbl.setText(calendarMaker.getString("pixels"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 4);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        boundsPnl.add(yUnitsLbl, gridBagConstraints);

        widthLbl.setText(calendarMaker.getString("Width_"));
        widthLbl.setDisplayedMnemonic(calendarMaker.getString( "Width_Mnemonic" ).charAt(0));
        widthLbl.setLabelFor(widthTxt);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        boundsPnl.add(widthLbl, gridBagConstraints);

        widthTxt.setColumns(6);
        widthTxt.setToolTipText(calendarMaker.getString("Width_Tooltip"));
        widthTxt.getAccessibleContext().setAccessibleDescription( calendarMaker.getString("Width_Tooltip") );
        widthTxt.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                widthTxtActionPerformed(evt);
            }
        });

        widthTxt.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                widthTxtFocusLost(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        boundsPnl.add(widthTxt, gridBagConstraints);

        widthUnitsLbl.setText(calendarMaker.getString("pixels"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 4);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        boundsPnl.add(widthUnitsLbl, gridBagConstraints);

        heightLbl.setText(calendarMaker.getString("Height_"));
        heightLbl.setDisplayedMnemonic(calendarMaker.getString( "Height_Mnemonic" ).charAt(0));
        heightLbl.setLabelFor(heightTxt);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 4, 2);
        boundsPnl.add(heightLbl, gridBagConstraints);

        heightTxt.setColumns(6);
        heightTxt.setToolTipText(calendarMaker.getString("Height_Tooltip"));
        heightTxt.getAccessibleContext().setAccessibleDescription( calendarMaker.getString("Height_Tooltip") );
        heightTxt.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                heightTxtActionPerformed(evt);
            }
        });

        heightTxt.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                heightTxtFocusLost(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 4, 2);
        boundsPnl.add(heightTxt, gridBagConstraints);

        heightUnitsLbl.setText(calendarMaker.getString("pixels"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 4, 4);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        boundsPnl.add(heightUnitsLbl, gridBagConstraints);

        constrainBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/icons/unlock.gif")));
        constrainBtn.setToolTipText(calendarMaker.getString("Constrain_Tooltip"));
        constrainBtn.setMargin(new java.awt.Insets(0, 0, 0, 0));
        constrainBtn.setSelectedIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/icons/lock.gif")));
        constrainBtn.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                constrainBtnActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridheight = 2;
        gridBagConstraints.insets = new java.awt.Insets(0, 8, 0, 4);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        boundsPnl.add(constrainBtn, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        add(boundsPnl, gridBagConstraints);

    }//GEN-END:initComponents

    private void constrainBtnActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_constrainBtnActionPerformed
    {//GEN-HEADEREND:event_constrainBtnActionPerformed
        updateConstrain();
    }//GEN-LAST:event_constrainBtnActionPerformed

    private void widthTxtFocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_widthTxtFocusLost
    {//GEN-HEADEREND:event_widthTxtFocusLost
        updateWidth();
    }//GEN-LAST:event_widthTxtFocusLost

    private void heightTxtFocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_heightTxtFocusLost
    {//GEN-HEADEREND:event_heightTxtFocusLost
        updateHeight();
    }//GEN-LAST:event_heightTxtFocusLost

    private void xTxtActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_xTxtActionPerformed
    {//GEN-HEADEREND:event_xTxtActionPerformed
        updateBounds();
    }//GEN-LAST:event_xTxtActionPerformed

    private void yTxtActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_yTxtActionPerformed
    {//GEN-HEADEREND:event_yTxtActionPerformed
        updateBounds();
    }//GEN-LAST:event_yTxtActionPerformed

    private void widthTxtActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_widthTxtActionPerformed
    {//GEN-HEADEREND:event_widthTxtActionPerformed
        updateWidth();
    }//GEN-LAST:event_widthTxtActionPerformed

    private void heightTxtActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_heightTxtActionPerformed
    {//GEN-HEADEREND:event_heightTxtActionPerformed
        updateHeight();
    }//GEN-LAST:event_heightTxtActionPerformed

    public void setObject(Object obj)
    {
        if( obj instanceof Component )
        {
            unregisterListeners();      // Unregister from previous object
            _object = (Component) obj;
            readBoundsAndConstrain();
            registerListeners();        // Register with new object
        }
    }    
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel yLbl;
    private javax.swing.JPanel boundsPnl;
    private javax.swing.JTextField yTxt;
    private javax.swing.JLabel widthUnitsLbl;
    private javax.swing.JToggleButton constrainBtn;
    private javax.swing.JLabel yUnitsLbl;
    private javax.swing.JLabel xUnitsLbl;
    private javax.swing.JLabel xLbl;
    private javax.swing.JTextField xTxt;
    private javax.swing.JLabel widthLbl;
    private javax.swing.JLabel heightLbl;
    private javax.swing.JLabel heightUnitsLbl;
    private javax.swing.JTextField widthTxt;
    private javax.swing.JTextField heightTxt;
    // End of variables declaration//GEN-END:variables

    
    /**
     * Read the bounds of the object and fill in the text boxes.
     */
    private void readBoundsAndConstrain()
    {
        if( _object != null )
        {
            Rectangle bounds = _object.getBounds();
            xTxt.setText( Integer.toString( bounds.x ) );
            yTxt.setText( Integer.toString( bounds.y ) );
            widthTxt.setText( Integer.toString( bounds.width ) );
            heightTxt.setText( Integer.toString( bounds.height ) );
                        
            if( _object instanceof JComponent )
            {
                Object obj = ((JComponent) _object).getClientProperty( DateablePanel.CONSTRAIN_PROPORTIONS );
                boolean constrain = false;
                
                if( (obj != null) && obj.equals( Boolean.TRUE ) )
                    constrain = true;
      
                constrainBtn.setEnabled( true );
                constrainBtn.setSelected( constrain );
            }
            else
            {
                constrainBtn.setEnabled( false );
                constrainBtn.setSelected( false );
            }
        }
    }
    
    
    /**
     * Write the values in the texts boxes back to the component.
     */
    private void updateBounds()
    {
        try
        {
            Rectangle bounds = new Rectangle();
            bounds.x = Integer.parseInt( xTxt.getText() );
            bounds.y = Integer.parseInt( yTxt.getText() );
            bounds.width = Integer.parseInt( widthTxt.getText() );
            bounds.height = Integer.parseInt( heightTxt.getText() );
            _object.setBounds( bounds );
            
            Container parent = _object.getParent();
            if( parent != null )
                parent.validate();
        }
        catch( NumberFormatException e ) {}
    }
    
    
    /**
     * Write the values in the texts boxes back to the component, constraining
     * the height if we have to.
     */
    private void updateWidth()
    {
        boolean constrain = constrainBtn.isSelected();
        Rectangle bounds = _object.getBounds();

        try
        {
            bounds.x = Integer.parseInt( xTxt.getText() );
            bounds.y = Integer.parseInt( yTxt.getText() );
            
            int newWidth = Integer.parseInt( widthTxt.getText() );
            
            if( constrain ) {
                double ratio = (bounds.width != 0) ? (double) bounds.height / (double) bounds.width : Double.MAX_VALUE;
                bounds.height = (int) (newWidth * ratio);
            } else
                bounds.height = Integer.parseInt( heightTxt.getText() );
            
            bounds.width = newWidth;
            
            _object.setBounds( bounds );
            
            Container parent = _object.getParent();
            if( parent != null )
                parent.validate();
        }
        catch( NumberFormatException e ) {}
    }

    
    /**
     * Write the values in the texts boxes back to the component, constraining
     * the width if we have to.
     */
    private void updateHeight()
    {
        boolean constrain = constrainBtn.isSelected();
        Rectangle bounds = _object.getBounds();

        try
        {
            bounds.x = Integer.parseInt( xTxt.getText() );
            bounds.y = Integer.parseInt( yTxt.getText() );
            
            int newHeight = Integer.parseInt( heightTxt.getText() );
            
            if( constrain ) {
                double ratio = (bounds.height != 0) ? (double) bounds.width / (double) bounds.height : Double.MAX_VALUE;
                bounds.width = (int) (newHeight * ratio);
            } else
                bounds.width = Integer.parseInt( widthTxt.getText() );
            
            bounds.height = newHeight;
            
            _object.setBounds( bounds );
            
            Container parent = _object.getParent();
            if( parent != null )
                parent.validate();
        }
        catch( NumberFormatException e ) {}
    }

    
    /**
     * Add or remove the CONSTRAIN_PROPORTIONS client property to/from the
     * object based on state of toggle-button.
     */
    private void updateConstrain()
    {
        if( _object instanceof JComponent )
        {
            JComponent jcmpnt = (JComponent) _object;

            if( constrainBtn.isSelected() )
                jcmpnt.putClientProperty( DateablePanel.CONSTRAIN_PROPORTIONS, Boolean.TRUE );
            else
                jcmpnt.putClientProperty( DateablePanel.CONSTRAIN_PROPORTIONS, null );
        }
    }
    
    
    /**
     * Register any listeners we need.  For the bounds of the component we need
     * to be a ComponentListener.
     */
    private void registerListeners()
    {
        _object.addComponentListener( this );
    }

    
    /**
     * Unregister any listeners we were using.
     */
    private void unregisterListeners()
    {
        if( _object != null )
        {
            _object.removeComponentListener( this );
        }
    }
    
    
    // From ComponentListener
    public void componentMoved( ComponentEvent e )
    {
        readBoundsAndConstrain();
    }
    
    // From ComponentListener
    public void componentResized( ComponentEvent e )
    {
        readBoundsAndConstrain();
    }
    
    // From ComponentListener
    public void componentHidden( ComponentEvent e )  {}

    // From ComponentListener
    public void componentShown( ComponentEvent e )  {}
    
    
    // From FocusListener
    public void focusGained( FocusEvent e )  {}
    
    // From FocusListener
    public void focusLost( FocusEvent e )
    {
        updateBounds();
    }
}
