/* *****************************************************************************
 *
 * Filename: AnchorEditor.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  29 January 2003
 *
 * Purpose:  Simple bean for selecting an anchor for GridBadConstraints.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.event.*;
import java.awt.GridBagConstraints;
import java.util.ResourceBundle;
import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.KeyStroke;
import javax.swing.JToggleButton;
import javax.swing.event.EventListenerList;

/**
 *
 * @author  Robert
 */
public class AnchorEditor
    extends EditorBase
{
    // Statics
    private static ResourceBundle resources = ResourceBundle.getBundle( "com/tsac/calmaker/beans/CalendarBeans" );
    
    // Properties
    private int _value = -1;
    
    
    /** Creates new form BeanForm */
    public AnchorEditor()
    {
        initComponents();
        
        // Add actions to set 'value' with appropriate anchor constant
        northWestBtn.addActionListener( new AnchorAction( GridBagConstraints.NORTHWEST ) );
        northBtn.addActionListener( new AnchorAction( GridBagConstraints.NORTH ) );
        northEastBtn.addActionListener( new AnchorAction( GridBagConstraints.NORTHEAST ) );
        
        westBtn.addActionListener( new AnchorAction( GridBagConstraints.WEST ) );
        centerBtn.addActionListener( new AnchorAction( GridBagConstraints.CENTER ) );
        eastBtn.addActionListener( new AnchorAction( GridBagConstraints.EAST ) );

        southWestBtn.addActionListener( new AnchorAction( GridBagConstraints.SOUTHWEST ) );
        southBtn.addActionListener( new AnchorAction( GridBagConstraints.SOUTH ) );
        southEastBtn.addActionListener( new AnchorAction( GridBagConstraints.SOUTHEAST ) );
        
        // Create actions for moving focus round the grid
        ActionMap actions = this.getActionMap();
        actions.put( "navigateLeft",  new GridNavigateAction( this, -1, 0 ) );
        actions.put( "navigateRight", new GridNavigateAction( this, +1, 0 ) );
        actions.put( "navigateUp",    new GridNavigateAction( this, 0, -1 ) );
        actions.put( "navigateDown",  new GridNavigateAction( this, 0, +1 ) );
        
        // Bind actions to cursor-keys 
        InputMap input = this.getInputMap( WHEN_ANCESTOR_OF_FOCUSED_COMPONENT );
        input.put( KeyStroke.getKeyStroke(KeyEvent.VK_LEFT,0), "navigateLeft" );
        input.put( KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT,0), "navigateRight" );
        input.put( KeyStroke.getKeyStroke(KeyEvent.VK_UP,0), "navigateUp" );
        input.put( KeyStroke.getKeyStroke(KeyEvent.VK_DOWN,0), "navigateDown" );
        
        // Add a FocusListener to move the initial focus onto the selected button
        this.addFocusListener( new FocusAdapter() {
            public void focusGained( FocusEvent e ) {
                JToggleButton button = buttonFromValue( _value );
                if( button != null )
                    button.requestFocus();
            }
        } );
        
        // Set up accessible names for all the buttons
        northWestBtn.getAccessibleContext().setAccessibleName( resources.getString( "Anchor_NorthWest_Name" ) );
        northBtn.getAccessibleContext().setAccessibleName( resources.getString( "Anchor_North_Name" ) );
        northEastBtn.getAccessibleContext().setAccessibleName( resources.getString( "Anchor_NorthEast_Name" ) );
        
        westBtn.getAccessibleContext().setAccessibleName( resources.getString( "Anchor_West_Name" ) );
        centerBtn.getAccessibleContext().setAccessibleName( resources.getString( "Anchor_Center_Name" ) );
        eastBtn.getAccessibleContext().setAccessibleName( resources.getString( "Anchor_East_Name" ) );

        southWestBtn.getAccessibleContext().setAccessibleName( resources.getString( "Anchor_SouthWest_Name" ) );
        southBtn.getAccessibleContext().setAccessibleName( resources.getString( "Anchor_South_Name" ) );
        southEastBtn.getAccessibleContext().setAccessibleName( resources.getString( "Anchor_SouthEast_Name" ) );
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents()//GEN-BEGIN:initComponents
    {
        anchorBtnGrp = new javax.swing.ButtonGroup();
        northWestBtn = new javax.swing.JToggleButton();
        northBtn = new javax.swing.JToggleButton();
        northEastBtn = new javax.swing.JToggleButton();
        westBtn = new javax.swing.JToggleButton();
        centerBtn = new javax.swing.JToggleButton();
        eastBtn = new javax.swing.JToggleButton();
        southWestBtn = new javax.swing.JToggleButton();
        southBtn = new javax.swing.JToggleButton();
        southEastBtn = new javax.swing.JToggleButton();

        setLayout(new java.awt.GridLayout(3, 3, 1, 1));

        northWestBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/nw.gif")));
        northWestBtn.setToolTipText(resources.getString("Anchor_NorthWest_Tooltip"));
        anchorBtnGrp.add(northWestBtn);
        northWestBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        add(northWestBtn);

        northBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/north.gif")));
        northBtn.setToolTipText(resources.getString("Anchor_North_Tooltip"));
        anchorBtnGrp.add(northBtn);
        northBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        add(northBtn);

        northEastBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/ne.gif")));
        northEastBtn.setToolTipText(resources.getString("Anchor_NorthEast_Tooltip"));
        anchorBtnGrp.add(northEastBtn);
        northEastBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        add(northEastBtn);

        westBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/west.gif")));
        westBtn.setToolTipText(resources.getString("Anchor_West_Tooltip"));
        anchorBtnGrp.add(westBtn);
        westBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        add(westBtn);

        centerBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/center.gif")));
        centerBtn.setToolTipText(resources.getString("Anchor_Center_Tooltip"));
        anchorBtnGrp.add(centerBtn);
        centerBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        add(centerBtn);

        eastBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/east.gif")));
        eastBtn.setToolTipText(resources.getString("Anchor_East_Tooltip"));
        anchorBtnGrp.add(eastBtn);
        eastBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        add(eastBtn);

        southWestBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/sw.gif")));
        southWestBtn.setToolTipText(resources.getString("Anchor_SouthWest_Tooltip"));
        anchorBtnGrp.add(southWestBtn);
        southWestBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        add(southWestBtn);

        southBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/south.gif")));
        southBtn.setToolTipText(resources.getString("Anchor_South_Tooltip"));
        anchorBtnGrp.add(southBtn);
        southBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        add(southBtn);

        southEastBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/se.gif")));
        southEastBtn.setToolTipText(resources.getString("Anchor_SouthEast_Tooltip"));
        anchorBtnGrp.add(southEastBtn);
        southEastBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        add(southEastBtn);

    }//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup anchorBtnGrp;
    private javax.swing.JToggleButton westBtn;
    private javax.swing.JToggleButton northWestBtn;
    private javax.swing.JToggleButton southWestBtn;
    private javax.swing.JToggleButton eastBtn;
    private javax.swing.JToggleButton northEastBtn;
    private javax.swing.JToggleButton northBtn;
    private javax.swing.JToggleButton southEastBtn;
    private javax.swing.JToggleButton southBtn;
    private javax.swing.JToggleButton centerBtn;
    // End of variables declaration//GEN-END:variables

    
    public void setValue( int value )
    {
        int oldval = _value;
        
        // De-bounce - otherwise we'll loop indefinitely
        if( value != oldval )
        {
            _value = value;
            
            JToggleButton button = buttonFromValue( value );
            if( button != null )
                button.setSelected( true );
                        
            firePropertyChange( "value", oldval, value );
        }
    }
    
    public int getValue()  { return _value; }
    
    
    /**
     * Re-usable code to find the appropriate button from an anchor constant.
     * Used in selecting and focusing buttons.  Will return null if an invalid
     * value is passed.  There is no need to map the other way as the
     * AnchorAction instances take care of it.
     */
    protected JToggleButton buttonFromValue( int value )
    {
        switch( value )
        {
        case GridBagConstraints.NORTHWEST: return northWestBtn;
        case GridBagConstraints.NORTH:     return northBtn;
        case GridBagConstraints.NORTHEAST: return northEastBtn;
        case GridBagConstraints.WEST:      return westBtn;
        case GridBagConstraints.CENTER:    return centerBtn;
        case GridBagConstraints.EAST:      return eastBtn;
        case GridBagConstraints.SOUTHWEST: return southWestBtn;
        case GridBagConstraints.SOUTH:     return southBtn;
        case GridBagConstraints.SOUTHEAST: return southEastBtn;
        }
        
        return null;
    }
    
    
    /**
     * Simple inner class to handle clicks on the buttons.  The value passed to
     * the constructor is passed to setValue().
     */
    private class AnchorAction
        implements ActionListener
    {
        private int _anchor;
        
        public AnchorAction( int anchor )
        {
            _anchor = anchor;
        }
        
        public void actionPerformed( ActionEvent e )
        {
            setValue( _anchor );
            fireActionEvent( new ActionEvent( this, ActionEvent.ACTION_PERFORMED, Integer.toString(_value) )  );
        }
    }
}
