/* *****************************************************************************
 *
 * Filename: GrayColorChooserPanel.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  09 February 2003
 *
 * Purpose:  A ColorChooser panel for picking grey values.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.Color;
import java.text.DecimalFormat;
import java.util.ResourceBundle;

/**
 *
 * @author  Robert
 */
public class GrayColorChooserPanel
    extends javax.swing.colorchooser.AbstractColorChooserPanel
{
    // Statics
    protected static final ResourceBundle resources = ResourceBundle.getBundle( "com/tsac/calmaker/beans/CalendarBeans" );
    protected static final DecimalFormat _formatter = new DecimalFormat( "##0.0" );
    
    
    /** Creates new form BeanForm */
    public GrayColorChooserPanel()
    {
        initComponents();

        // Make slider show "mercury" like a thermometer
        graySlider.putClientProperty( "JSlider.isFilled", Boolean.TRUE );
        
        // Accessibility Stuff
        whiteLbl.getAccessibleContext().setAccessibleName( resources.getString( "White_Name" ) );
        graySlider.getAccessibleContext().setAccessibleName( resources.getString( "Gray_Slider_Name" ) );
        blackLbl.getAccessibleContext().setAccessibleName( resources.getString( "Black_Name" ) );
        percentTxt.getAccessibleContext().setAccessibleName( resources.getString( "Gray_Percent_Name" ) );
        percentTxt.getAccessibleContext().setAccessibleDescription( resources.getString( "Gray_Percent_Tooltip" ) );
        redTxt.getAccessibleContext().setAccessibleName( resources.getString( "Red_Name" ) );
        redTxt.getAccessibleContext().setAccessibleDescription( resources.getString( "Red_Tooltip" ) );
        greenTxt.getAccessibleContext().setAccessibleName( resources.getString( "Green_Name" ) );
        greenTxt.getAccessibleContext().setAccessibleDescription( resources.getString( "Green_Tooltip" ) );
        blueTxt.getAccessibleContext().setAccessibleName( resources.getString( "Blue_Name" ) );
        blueTxt.getAccessibleContext().setAccessibleDescription( resources.getString( "Blue_Tooltip" ) );
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents()//GEN-BEGIN:initComponents
    {
        java.awt.GridBagConstraints gridBagConstraints;

        sliderPnl = new javax.swing.JPanel();
        whiteLbl = new javax.swing.JLabel();
        graySlider = new javax.swing.JSlider();
        blackLbl = new javax.swing.JLabel();
        percentTxt = new javax.swing.JTextField();
        percentLbl = new javax.swing.JLabel();
        rgbPnl = new javax.swing.JPanel();
        redLbl = new javax.swing.JLabel();
        redTxt = new javax.swing.JTextField();
        greenLbl = new javax.swing.JLabel();
        greenTxt = new javax.swing.JTextField();
        blueLbl = new javax.swing.JLabel();
        blueTxt = new javax.swing.JTextField();

        setLayout(new java.awt.GridBagLayout());

        sliderPnl.setLayout(new java.awt.GridBagLayout());

        whiteLbl.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/white.gif")));
        whiteLbl.setToolTipText(resources.getString("White_Tooltip"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 0);
        sliderPnl.add(whiteLbl, gridBagConstraints);

        graySlider.setMajorTickSpacing(85);
        graySlider.setMaximum(255);
        graySlider.setMinorTickSpacing(17);
        graySlider.setPaintTicks(true);
        graySlider.setToolTipText(resources.getString("Gray_Slider_Tooltip"));
        graySlider.setValue(128);
        graySlider.setPreferredSize(new java.awt.Dimension(210, 42));
        graySlider.addChangeListener(new javax.swing.event.ChangeListener()
        {
            public void stateChanged(javax.swing.event.ChangeEvent evt)
            {
                graySliderStateChanged(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 0, 4, 0);
        sliderPnl.add(graySlider, gridBagConstraints);

        blackLbl.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/black.gif")));
        blackLbl.setToolTipText(resources.getString("Black_Tooltip"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 0, 4, 4);
        sliderPnl.add(blackLbl, gridBagConstraints);

        percentTxt.setColumns(5);
        percentTxt.setToolTipText(resources.getString("Gray_Percent_Tooltip"));
        percentTxt.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                percentTxtActionPerformed(evt);
            }
        });

        percentTxt.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                percentTxtFocusLost(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 2);
        sliderPnl.add(percentTxt, gridBagConstraints);

        percentLbl.setText("%");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 2, 4, 4);
        sliderPnl.add(percentLbl, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.SOUTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(48, 0, 0, 0);
        add(sliderPnl, gridBagConstraints);

        rgbPnl.setLayout(new java.awt.GridBagLayout());

        redLbl.setText(resources.getString("Red"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 2);
        rgbPnl.add(redLbl, gridBagConstraints);

        redTxt.setColumns(3);
        redTxt.setEditable(false);
        redTxt.setToolTipText(resources.getString("Red_Tooltip"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 2, 0, 8);
        rgbPnl.add(redTxt, gridBagConstraints);

        greenLbl.setText(resources.getString("Green"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 8, 0, 2);
        rgbPnl.add(greenLbl, gridBagConstraints);

        greenTxt.setColumns(3);
        greenTxt.setEditable(false);
        greenTxt.setToolTipText(resources.getString("Green_Tooltip"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 2, 0, 8);
        rgbPnl.add(greenTxt, gridBagConstraints);

        blueLbl.setText(resources.getString("Blue"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 8, 0, 2);
        rgbPnl.add(blueLbl, gridBagConstraints);

        blueTxt.setColumns(3);
        blueTxt.setEditable(false);
        blueTxt.setToolTipText(resources.getString("Blue_Tooltip"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 2, 0, 0);
        rgbPnl.add(blueTxt, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new java.awt.Insets(48, 0, 0, 0);
        add(rgbPnl, gridBagConstraints);

    }//GEN-END:initComponents

    private void percentTxtFocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_percentTxtFocusLost
    {//GEN-HEADEREND:event_percentTxtFocusLost
        actionPercentChanged();
    }//GEN-LAST:event_percentTxtFocusLost

    private void percentTxtActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_percentTxtActionPerformed
    {//GEN-HEADEREND:event_percentTxtActionPerformed
        actionPercentChanged();
    }//GEN-LAST:event_percentTxtActionPerformed

    private void graySliderStateChanged(javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_graySliderStateChanged
    {//GEN-HEADEREND:event_graySliderStateChanged
        actionSliderChanged();
    }//GEN-LAST:event_graySliderStateChanged

    protected void buildChooser()
    {
    }    
    
    public String getDisplayName()
    {
        return resources.getString( "Gray" );
    }    
    
    public javax.swing.Icon getLargeDisplayIcon()
    {
        return null;
    }
    
    public javax.swing.Icon getSmallDisplayIcon()
    {
        return null;
    }

    
    public void updateChooser()
    {
        Color c = getColorSelectionModel().getSelectedColor();
        int r = c.getRed();
        int g = c.getGreen();
        int b = c.getBlue();
        
        int newval = 255 - (int) Math.round( 0.3*r + 0.59*g + 0.11*b );
        int oldval = graySlider.getValue();
        
        if( newval != oldval )
            graySlider.setValue( newval );
    }

    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel blueLbl;
    private javax.swing.JLabel blackLbl;
    private javax.swing.JTextField blueTxt;
    private javax.swing.JLabel greenLbl;
    private javax.swing.JLabel percentLbl;
    private javax.swing.JTextField greenTxt;
    private javax.swing.JTextField percentTxt;
    private javax.swing.JLabel redLbl;
    private javax.swing.JSlider graySlider;
    private javax.swing.JPanel rgbPnl;
    private javax.swing.JTextField redTxt;
    private javax.swing.JLabel whiteLbl;
    private javax.swing.JPanel sliderPnl;
    // End of variables declaration//GEN-END:variables
    

    protected void updatePercent()
    {
        // Convert slider to percentage with 1 decimal place (x10)
        double val = Math.round( graySlider.getValue() / 0.255 );
        percentTxt.setText( _formatter.format( val / 10 ) );
    }
    
    
    protected void actionPercentChanged()
    {
        try
        {
            double val = 100 - Double.parseDouble( percentTxt.getText() );
            int gray = (int) Math.round( val * 2.55 );
            getColorSelectionModel().setSelectedColor( new Color( gray, gray, gray ) );
        }
        catch( NumberFormatException e ) {}
        
        updatePercent();
        updateRGB();        
    }
    
    
    protected void actionSliderChanged()
    {
        int gray = 255 - graySlider.getValue();
        getColorSelectionModel().setSelectedColor( new Color( gray, gray, gray ) );
        updatePercent();
        updateRGB();
    }
    
    
    protected void updateRGB()
    {
        int gray = 255 - graySlider.getValue();
        redTxt.setText( Integer.toString( gray ) );
        greenTxt.setText( Integer.toString( gray ) );
        blueTxt.setText( Integer.toString( gray ) );
    }
}
