/* *****************************************************************************
 *
 * Filename: MonthImageCellEditorComponent.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  20 December 2002, 14:27
 *
 * Purpose:  CellEditor Component class used in MonthImageCustomizer for editing
 *           image filenames.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import javax.swing.JFileChooser;
import javax.swing.CellEditor;
import java.awt.Font;
import java.awt.event.FocusEvent;

/**
 *
 * @author  Robert
 */
public class MonthImageCellEditorComponent
    extends javax.swing.JPanel
    implements java.awt.event.FocusListener
{
    private CellEditor _editor;
    
    
    /** Creates new form MonthImageCellEditor */
    public MonthImageCellEditorComponent( CellEditor editor )
    {
        _editor = editor;
        
        initComponents();
        
        filenameTxt.addFocusListener( this );
        chooserBtn.addFocusListener( this );
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents()//GEN-BEGIN:initComponents
    {
        java.awt.GridBagConstraints gridBagConstraints;

        filenameTxt = new javax.swing.JTextField();
        chooserBtn = new javax.swing.JButton();

        setLayout(new java.awt.GridBagLayout());

        setBackground((java.awt.Color) javax.swing.UIManager.getDefaults().get("Table.focusCellBackground"));
        filenameTxt.setBorder(null);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(0, 1, 1, 1);
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        add(filenameTxt, gridBagConstraints);

        chooserBtn.setText("...");
        chooserBtn.setMargin(new java.awt.Insets(2, 4, 2, 4));
        chooserBtn.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                chooserBtnActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.weighty = 1.0;
        add(chooserBtn, gridBagConstraints);

    }//GEN-END:initComponents

    private void chooserBtnActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_chooserBtnActionPerformed
    {//GEN-HEADEREND:event_chooserBtnActionPerformed
        JFileChooser opendlg = new JFileChooser( filenameTxt.getText().length() != 0 ? filenameTxt.getText() : "." );
        javax.swing.filechooser.FileFilter imgFilter;
        opendlg.addChoosableFileFilter( imgFilter = new javax.swing.filechooser.FileFilter() {
            public boolean accept( java.io.File f ) {
                String s = f.getName().toLowerCase();
                return s.endsWith( ".jpg" ) || s.endsWith( ".jpeg" ) || s.endsWith( ".gif" ) || f.isDirectory();
            }
            public String getDescription() {
                return java.util.ResourceBundle.getBundle("com/tsac/calmaker/beans/CalendarBeans").getString("Image_files_filter");
            }
        });
        opendlg.setFileFilter( imgFilter );

        if( opendlg.showOpenDialog( this ) == JFileChooser.APPROVE_OPTION )
        {
            filenameTxt.setText( opendlg.getSelectedFile().toString() );
            _editor.stopCellEditing();
        }
    }//GEN-LAST:event_chooserBtnActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextField filenameTxt;
    private javax.swing.JButton chooserBtn;
    // End of variables declaration//GEN-END:variables
    

    public void setFilename( String filename )
    {
        filenameTxt.setText( filename );
    }
    
    public String getFilename()
    {
        return filenameTxt.getText();
    }
    
    
    // From JComponent
    public void setFont( Font font )
    {
        super.setFont( font );
        
        // Odd problem with this being called by installUI from JPanel() before created!
        if( filenameTxt != null )
            filenameTxt.setFont( font );
        
        if( chooserBtn != null )
            chooserBtn.setFont( font );
    }
    
    
    public void focusGained( FocusEvent e )
    {
        repaint();
    }
    
    public void focusLost( FocusEvent e )
    {
    }
}
